<?php
/**
 * @file
 * Admin page callbacks.
 */

/**
 * Form function for options.
 *
 * @ingroup forms
 */
function manymail_config_options_form($form, &$form_state) {
  $form['throttle'] = array(
    '#type' => 'fieldset',
    '#title' => t('E-mail throttling'),
    '#description' => t('This prevents server time-outs when trying to send a lot of e-mails and makes it less likely for your mail to be marked as spam.'),
    '#weight' => 1,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['throttle']['manymail_options_throttle_pause'] = array(
    '#type' => 'textfield',
    '#title' => t('Pause between batches'),
    '#default_value' => variable_get('manymail_options_throttle_pause', 5),
    '#description' => t('The amount of seconds to wait between sending batches.'),
  );

  $form['throttle']['manymail_options_throttle_amount'] = array(
    '#type' => 'textfield',
    '#title' => t('E-mails per batch'),
    '#default_value' => variable_get('manymail_options_throttle_amount', 50),
    '#description' => t('The amount of e-mails to send per batch.'),
  );

  $form['content'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content encoding'),
    '#weight' => 2,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['content']['manymail_options_charset'] = array(
    '#type' => 'textfield',
    '#title' => t('Character set'),
    '#default_value' => variable_get('manymail_options_charset', 'utf-8'),
    '#description' => t('The character set to encode your e-mails with.<br />Common options are utf-8 (recommended) and iso-8859-1.'),
  );

  $form['content']['manymail_options_html_to_text'] = array(
    '#type' => 'checkbox',
    '#title' => t('Convert HTML to marked up plain text'),
    '#default_value' => variable_get('manymail_options_html_to_text', 0),
    '#description' => t('When checked, runs the subject and message body through <a href="@apiref" target="_blank">drupal_html_to_text</a>.', array('@apiref' => 'http://api.drupal.org/api/drupal/includes!mail.inc/function/drupal_html_to_text/7')),
  );

  $form['logging'] = array(
    '#type' => 'fieldset',
    '#title' => t('Logging options'),
    '#weight' => 3,
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $form['logging']['manymail_options_log_per_mail'] = array(
    '#type' => 'checkbox',
    '#title' => t('Log successfully sent e-mails on a per-mail base.'),
    '#default_value' => variable_get('manymail_options_log_per_mail', 1),
    '#description' => t('Having this checked will guarantee that a cancelled e-mailing process will be resumed at the user it last tried to mail.<br />Unchecking this will make ManyMail only log on a per-batch level, which results in slightly less database load.<br />Note: this could lead to some people receiving the e-mail twice after a resume action.'),
  );

  return system_settings_form($form);
}

/**
 * Implements validation from the Form API.
 *
 * Verifies whether the throttle delay and amount are valid numbers
 * and converts them to integers if they are.
 *
 * @see manymail_config_options_form()
 */
function manymail_config_options_form_validate($form, &$form_state) {
  if (!is_numeric($form_state['values']['manymail_options_throttle_pause']) || $form_state['values']['manymail_options_throttle_pause'] < 1) {
    form_set_error('manymail_options_throttle_pause', t('You must enter a positive number for the throttle delay.'));
  }
  else {
    form_set_value($form['throttle']['manymail_options_throttle_pause'], floor($form_state['values']['manymail_options_throttle_pause']), $form_state);
  }

  if (!is_numeric($form_state['values']['manymail_options_throttle_amount']) || $form_state['values']['manymail_options_throttle_amount'] < 1) {
    form_set_error('manymail_options_throttle_amount', t('You must enter a positive number for the throttle e-mail amount.'));
  }
  else {
    form_set_value($form['throttle']['manymail_options_throttle_amount'], floor($form_state['values']['manymail_options_throttle_amount']), $form_state);
  }
}

/**
 * Form function for content defaults.
 *
 * @ingroup forms
 */
function manymail_config_defaults_form($form, &$form_state) {
  $form['default_content'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content'),
    '#weight' => 2,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['default_content']['manymail_default_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => variable_get('manymail_default_subject', t('ManyMail e-mail')),
    '#description' => t('The e-mail subject.'),
  );

  $form['default_content']['manymail_default_body'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#default_value' => variable_get('manymail_default_body', ''),
    '#description' => t('The message body.'),
  );

  $form['default_content']['manymail_default_signature'] = array(
    '#type' => 'textarea',
    '#title' => t('Signature'),
    '#default_value' => variable_get('manymail_default_signature', "Yours sincerely,\nThe site admin"),
    '#description' => t('Gets appended to every message. Can be disabled in the newsletter itself.'),
  );

  return system_settings_form($form);
}

/**
 * Form function for header defaults.
 *
 * @ingroup forms
 */
function manymail_config_headers_form($form, &$form_state) {
  $form['default_from'] = array(
    '#type' => 'fieldset',
    '#title' => t('From: header'),
    '#weight' => 1,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['default_from']['manymail_default_from'] = array(
    '#type' => 'textfield',
    '#title' => t('From address'),
    '#default_value' => variable_get('manymail_default_from', variable_get('site_mail', 'admin@example.com')),
    '#description' => t('The e-mail address that e-mails appear to be sent from.'),
  );

  $form['default_from']['manymail_default_from_name'] = array(
    '#type' => 'textfield',
    '#title' => t('From name'),
    '#default_value' => variable_get('manymail_default_from_name', 'Site admin'),
    '#description' => t('The name of the person that e-mails appear to be sent from.'),
  );

  $form['default_reply_to'] = array(
    '#type' => 'fieldset',
    '#title' => t('Reply-to: header'),
    '#weight' => 2,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['default_reply_to']['manymail_default_reply_to'] = array(
    '#type' => 'textfield',
    '#title' => t('Reply-to address'),
    '#default_value' => variable_get('manymail_default_reply_to', ''),
    '#description' => t('The e-mail address you want recipients to reply to.<br />Leave empty if you want this to be the same as the <em>From</em> address.'),
  );

  $form['default_reply_to']['manymail_default_reply_to_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Reply-to name'),
    '#default_value' => variable_get('manymail_default_reply_to_name', ''),
    '#description' => t('The name of the person for the reply-to address.<br />Leave empty if you want this to be the same as the <em>From</em> name.'),
  );

  $form['other_address_headers'] = array(
    '#type' => 'fieldset',
    '#title' => t('Other address headers'),
    '#weight' => 3,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['other_address_headers']['manymail_options_sender_address'] = array(
    '#type' => 'textfield',
    '#title' => t('Sender address'),
    '#default_value' => variable_get('manymail_options_sender_address', ''),
    '#description' => t('The e-mail address that you are actually sending from.<br />If you set a <em>From</em> address that does not match the actual sender address, you should fill this out to reduce the likelihood of your message being discarded as spam.'),
  );

  $form['other_address_headers']['manymail_options_return_path_address'] = array(
    '#type' => 'textfield',
    '#title' => t('Return-path address'),
    '#default_value' => variable_get('manymail_options_return_path_address', ''),
    '#description' => t('Also called the <em>Bounce address</em>.<br />If you want to process bounced e-mails, you can specify the address you want those mails to go to here.'),
  );

  return system_settings_form($form);
}

/**
 * Form function for SMTP configuration.
 *
 * @ingroup forms
 */
function manymail_config_smtp_form($form, &$form_state) {
  // Show a warning when the SMTP settings don't add up.
  if (!manymail_validate_smtp()) {
    drupal_set_message(t('Could not connect to the SMTP host. Please verify the settings below.'), 'error');
  }

  $form['server'] = array(
    '#type' => 'fieldset',
    '#title' => t('Server settings'),
    '#weight' => 1,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['server']['manymail_smtp_host'] = array(
    '#type' => 'textfield',
    '#title' => t('Host'),
    '#default_value' => variable_get('manymail_smtp_host', 'localhost'),
    '#description' => t('The server to send mails through.'),
  );

  $form['server']['manymail_smtp_port'] = array(
    '#type' => 'textfield',
    '#title' => t('Port'),
    '#default_value' => variable_get('manymail_smtp_port', 25),
    '#description' => t('The SMTP port for the mail server.'),
  );

  $form['server']['manymail_smtp_timeout'] = array(
    '#type' => 'textfield',
    '#title' => t('Timeout'),
    '#default_value' => variable_get('manymail_smtp_timeout', 10),
    '#description' => t('Maximum amount of seconds to wait for the server to respond.'),
  );

  $form['security'] = array(
    '#attributes' => array(
      'id' => 'smtp-security-fieldset',
    ),
    '#type' => 'fieldset',
    '#title' => t('Security settings'),
    '#weight' => 2,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['security']['manymail_smtp_secure'] = array(
    '#type' => 'radios',
    '#title' => t('Server prefix'),
    '#options' => array(
      'none' => '&lt;none&gt;',
      'ssl' => 'ssl',
      'tls' => 'tls',
    ),
    '#default_value' => variable_get('manymail_smtp_secure', 'none'),
    '#description' => t('The connection prefix to the server.'),
  );

  $smtp_auth = variable_get('manymail_smtp_auth', 0);
  $smtp_auth_set = isset($form_state['values']['manymail_smtp_auth']);

  $form['security']['manymail_smtp_auth'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use SMTP authentication'),
    '#default_value' => $smtp_auth,
    '#description' => t('You will be asked to provide a username and password if you check this.'),
    '#ajax' => array(
      'callback' => '_manymail_config_smtp_form_ajax',
      'wrapper' => 'smtp-security-fieldset',
      'method' => 'replace',
      'effect' => 'fade',
    ),
  );

  // If the checkbox was altered: respect that choice.
  // Otherwise respect the previously saved settings.
  if (($smtp_auth_set && $form_state['values']['manymail_smtp_auth'] == 1) || (!$smtp_auth_set && !empty($smtp_auth))) {
    unset($form['security']['manymail_smtp_auth']['#description']);

    $form['security']['credentials'] = array(
      '#type' => 'fieldset',
      '#title' => t('Credentials'),
      '#collapsible' => FALSE,
    );

    $form['security']['credentials']['manymail_smtp_username'] = array(
      '#type' => 'textfield',
      '#title' => t('Username'),
      '#default_value' => variable_get('manymail_smtp_username', ''),
      '#required' => TRUE,
    );

    $form['security']['credentials']['manymail_smtp_password'] = array(
      '#type' => 'password',
      '#title' => t('Password'),
      '#default_value' => variable_get('manymail_smtp_password', ''),
      '#required' => TRUE,
    );
  }

  return system_settings_form($form);
}

/**
 * AJAX handler for the SMTP configuration.
 *
 * Adds the SMTP credentials part of the form.
 *
 * @see manymail_config_smtp_form()
 */
function _manymail_config_smtp_form_ajax($form, &$form_state) {
  return $form['security'];
}

/**
 * Form function for target roles.
 *
 * @ingroup forms
 */
function manymail_targets_roles_form($form, &$form_state) {
  // Make the first $roles checkbox a select-all.
  $js_setting['ManyMailCheckAll'] = array(
    'list' => '#edit-manymail-allowed-roles',
    'key' => '#edit-manymail-allowed-roles-' . DRUPAL_AUTHENTICATED_RID,
  );
  drupal_add_js($js_setting, 'setting');
  drupal_add_js(drupal_get_path('module', 'manymail') . '/misc/checkall.js');

  $form['allowed_roles'] = array(
    '#type' => 'fieldset',
    '#title' => t('Allowed recipient roles'),
    '#description' => t('The roles that may be sent a newsletter.'),
    '#weight' => 1,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  // Select all but anonymous role.
  $roles = user_roles(TRUE);

  array_walk($roles, 'check_plain');
  $form['allowed_roles']['manymail_allowed_roles'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Target roles'),
    '#title_display' => 'invisible',
    '#options' => $roles,
    '#default_value' => variable_get('manymail_allowed_roles', $roles),
  );

  $form['allowed_roles']['manymail_allowed_roles_override'] = array(
    '#type' => 'checkbox',
    '#title' => t('Certain users may ignore this restriction'),
    '#default_value' => variable_get('manymail_allowed_roles_override', 1),
    '#description' => t('When checked, users with the %permission permission can target any role regardless of what has been configured above.', array('%permission' => t('Override preset recipient roles'))),
  );

  return system_settings_form($form);
}
